@@ -3,7 +3,7 @@
  *
  * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
- * Copyright 2004 Dag Lem <resid@nimrod.no>
+ * Copyright 2004,2010 Dag Lem <resid@nimrod.no>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,15 +31,11 @@ namespace reSIDfp
 /**
  * A 15 bit LFSR is used to implement the envelope rates, in effect dividing
  * the clock to the envelope counter by the currently selected rate period.
- * <P>
+ * <p>
  * In addition, another counter is used to implement the exponential envelope decay,
  * in effect further dividing the clock to the envelope counter.
  * The period of this counter is set to 1, 2, 4, 8, 16, 30 at the envelope counter
  * values 255, 93, 54, 26, 14, 6, respectively.
- *
- * @author Ken Händel
- * @author Dag Lem
- * @author Antti Lankila
  */
 class EnvelopeGenerator
 {
@@ -53,14 +49,10 @@ private:
         ATTACK, DECAY_SUSTAIN, RELEASE
     };
 
-    /**
-     * XOR shift register for ADSR prescaling
-     */
+    /// XOR shift register for ADSR prescaling.
     int lfsr;
 
-    /**
-     * Comparison value (period) of the rate counter before next event.
-     */
+    /// Comparison value (period) of the rate counter before next event.
     int rate;
 
     /**
@@ -75,32 +67,30 @@ private:
      */
     int exponential_counter_period;
 
-    /** Attack register */
+    /// Attack register
     int attack;
 
-    /** Decay register */
+    /// Decay register
     int decay;
 
-    /** Sustain register */
+    /// Sustain register
     int sustain;
 
-    /** Release register */
+    /// Release register
     int release;
 
-    /** Current envelope state */
+    /// Current envelope state
     State state;
 
-    /**
-     * Whether hold is enabled. Only switching to ATTACK can release envelope.
-     */
+    /// Whether hold is enabled. Only switching to ATTACK can release envelope.
     bool hold_zero;
 
     bool envelope_pipeline;
 
-    /** Gate bit */
+    /// Gate bit
     bool gate;
 
-    /** The current digital value of envelope output. */
+    /// The current digital value of envelope output.
     unsigned char envelope_counter;
 
     /**
@@ -110,12 +100,10 @@ private:
      */
     short dac[256];
 
-    void set_exponential_counter();
-
     /**
      * Lookup table to convert from attack, decay, or release value to rate
      * counter period.
-     * <P>
+     * <p>
      * The rate counter is a 15 bit register which is left shifted each cycle.
      * When the counter reaches a specific comparison value,
      * the envelope counter is incremented (attack) or decremented
@@ -125,6 +113,9 @@ private:
      */
     static const int adsrtable[16];
 
+private:
+    void set_exponential_counter();
+
 public:
     /**
      * Set chip model.
@@ -136,7 +127,7 @@ public:
     void setChipModel(ChipModel chipModel);
 
     /**
-     * SID clocking - 1 cycle.
+     * SID clocking.
      */
     void clock();
 
@@ -170,23 +161,25 @@ public:
      */
     void reset();
 
-    // ----------------------------------------------------------------------------
-    // Register functions.
-    // ----------------------------------------------------------------------------
-
     /**
+     * Write control register.
+     *
      * @param control
      *            control register
      */
     void writeCONTROL_REG(unsigned char control);
 
     /**
+     * Write Attack/Decay register.
+     *
      * @param attack_decay
      *            attack/decay value
      */
     void writeATTACK_DECAY(unsigned char attack_decay);
 
     /**
+     * Write Sustain/Release register.
+     *
      * @param sustain_release
      *            sustain/release value
      */
@@ -210,7 +203,7 @@ namespace reSIDfp
 RESID_INLINE
 void EnvelopeGenerator::clock()
 {
-    if (envelope_pipeline)
+    if (unlikely(envelope_pipeline))
     {
         --envelope_counter;
         envelope_pipeline = false;
@@ -229,7 +222,7 @@ void EnvelopeGenerator::clock()
     // so the ADSR delay bug should be correcly modeled
 
     // check to see if LFSR matches table value
-    if (lfsr != rate)
+    if (likely(lfsr != rate))
     {
         // it wasn't a match, clock the LFSR once
         // by performing XOR on last 2 bits
@@ -250,7 +243,7 @@ void EnvelopeGenerator::clock()
         exponential_counter = 0;
 
         // Check whether the envelope counter is frozen at zero.
-        if (hold_zero)
+        if (unlikely(hold_zero))
         {
             return;
         }
@@ -265,7 +258,7 @@ void EnvelopeGenerator::clock()
             //
             ++envelope_counter;
 
-            if (envelope_counter == (unsigned char) 0xff)
+            if (unlikely(envelope_counter == (unsigned char) 0xff))
             {
                 state = DECAY_SUSTAIN;
                 rate = adsrtable[decay];
@@ -281,7 +274,7 @@ void EnvelopeGenerator::clock()
             //
             // For a detailed description see:
             // http://ploguechipsounds.blogspot.it/2010/11/new-research-on-sid-adsr.html
-            if (envelope_counter == (unsigned char)(sustain << 4 | sustain))
+            if (likely(envelope_counter == (unsigned char)(sustain << 4 | sustain)))
             {
                 return;
             }
@@ -296,7 +289,7 @@ void EnvelopeGenerator::clock()
             // This has been verified by sampling ENV3.
             // NB! The operation below requires two's complement integer.
             //
-            if (exponential_counter_period != 1)
+            if (unlikely(exponential_counter_period != 1))
             {
                 // The decrement is delayed one cycle.
                 envelope_pipeline = true;
