@@ -1,9 +1,9 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2014 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
- * Copyright 2004 Dag Lem <resid@nimrod.no>
+ * Copyright 2004,2010 Dag Lem <resid@nimrod.no>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
 #define FILTER8580_H
 
 #include <cmath>
+#include <cstring>
+
+#include <stdint.h>
 
 #include "siddefs-fp.h"
 
@@ -32,18 +35,41 @@
 namespace reSIDfp
 {
 
+/**
+ * Simple white noise generator.
+ * Generates small low quality pseudo random numbers
+ * useful to prevent float denormals.
+ *
+ * Based on the paper "Denormal numbers in floating point signal
+ * processing applications" from Laurent de Soras
+ * http://ldesoras.free.fr/prod.html#doc_denormal
+ */
+class antiDenormalNoise
+{
+private:
+    uint32_t rand_state;
+
+public:
+    antiDenormalNoise() :
+        rand_state(1) {}
+
+    inline float get()
+    {
+        // FIXME
+        // This code assumes IEEE-754 floating point representation
+        // and same endianness for integers and floats
+        rand_state = rand_state * 1234567UL + 890123UL;
+        const uint32_t mantissa = rand_state & 0x807F0000; // Keep only most significant bits
+        const uint32_t flt_rnd = mantissa | 0x1E000000; // Set exponent
+        float temp;
+        memcpy(&temp, &flt_rnd, sizeof(float));
+        return temp;
+    }
+};
+
 /**
  * Filter for 8580 chip based on simple linear approximation
  * of the FC control.
- *
- * This is like the original reSID filter except the phase
- * of BP output has been inverted. I saw samplings on the internet
- * that indicated it would genuinely happen like this.
- *
- * @author Ken Händel
- * @author Dag Lem
- * @author Antti Lankila
- * @author Leandro Nini
  */
 class Filter8580 : public Filter
 {
@@ -53,6 +79,8 @@ private:
     float w0, _1_div_Q;
     int ve;
 
+    antiDenormalNoise noise;
+
 public:
     Filter8580() :
         highFreq(12500.),
@@ -65,14 +93,29 @@ public:
 
     int clock(int voice1, int voice2, int voice3);
 
+    /**
+     * Set filter cutoff frequency.
+     */
     void updatedCenterFrequency() { w0 = (float)(2. * M_PI * highFreq * fc / 2047. / 1e6); }
 
-    void updatedResonance() { _1_div_Q = 1.f / (0.707f + res / 15.f); }
+    /**
+     * Set filter resonance.
+     *
+     * The following function for 1/Q has been modeled in the MOS 8580:
+     *
+     * 1/Q = 2^(1/2)*2^(-x/8) = 2^(1/2 - x/8) = 2^((4 - x)/8)
+     */
+    void updatedResonance() { _1_div_Q = (float)pow(2., (4 - res) / 8.); }
 
     void input(int input) { ve = input << 4; }
 
     void updatedMixing() {}
 
+    /**
+     * Set filter curve type based on single parameter.
+     *
+     * @param curvePosition filter's center frequency expressed in Hertz, default is 12500
+     */
     void setFilterCurve(double curvePosition) { highFreq = curvePosition; }
 };
 
@@ -80,8 +123,7 @@ public:
 
 #if RESID_INLINING || defined(FILTER8580_CPP)
 
-#include <stdlib.h>
-#include <math.h>
+#include <cassert>
 
 namespace reSIDfp
 {
@@ -113,11 +155,13 @@ int Filter8580::clock(int voice1, int voice2, int voice3)
 
     (filtE ? Vi : Vo) += ve;
 
-    const float dVbp = w0 * Vhp;
-    const float dVlp = w0 * Vbp;
-    Vbp -= dVbp;
-    Vlp -= dVlp;
-    Vhp = (Vbp * _1_div_Q) - Vlp - Vi + float(rand()) / float(RAND_MAX);
+    Vlp -= w0 * Vbp;
+    Vbp -= w0 * Vhp;
+    Vhp = (Vbp * _1_div_Q) - Vlp - Vi + noise.get();
+
+    assert(std::fpclassify(Vlp) != FP_SUBNORMAL);
+    assert(std::fpclassify(Vbp) != FP_SUBNORMAL);
+    assert(std::fpclassify(Vhp) != FP_SUBNORMAL);
 
     float Vof = (float)Vo;
 
