@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2014 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,8 @@
 
 #include <cmath>
 
+#include "siddefs-fp.h"
+
 namespace reSIDfp
 {
 
@@ -36,8 +38,9 @@ double OpAmp::solve(double n, double vi)
     double bk = vmax;
 
     const double a = n + 1.;
-    const double b = Vddt;
-    const double b_vi = (b - vi);
+    const double b = kVddt;
+    double b_vi = (b - vi);
+    if (b_vi < 0.) b_vi = 0.;
     const double c = n * (b_vi * b_vi);
 
     for (;;)
@@ -45,28 +48,37 @@ double OpAmp::solve(double n, double vi)
         const double xk = x;
 
         // Calculate f and df.
+
+        Spline::Point out;
+
         opamp->evaluate(x, out);
-        const double vo = out[0];
-        const double dvo = out[1];
+        const double vo = out.x;
+        const double dvo = out.y;
 
-        const double b_vx = b - x;
-        const double b_vo = b - vo;
+        double b_vx = b - x;
+        if (b_vx < 0.) b_vx = 0.;
+        double b_vo = b - vo;
+        if (b_vo < 0.) b_vo = 0.;
 
+        // f = a*(b - vx)^2 - c - (b - vo)^2
         const double f = a * (b_vx * b_vx) - c - (b_vo * b_vo);
+
+        // df = 2*((b - vo)*dvo - a*(b - vx))
         const double df = 2. * (b_vo * dvo - a * b_vx);
 
+        // Newton-Raphson step: xk1 = xk - f(xk)/f'(xk)
         x -= f / df;
 
-        if (fabs(x - xk) < EPSILON)
+        if (unlikely(fabs(x - xk) < EPSILON))
         {
             opamp->evaluate(x, out);
-            return out[0];
+            return out.x;
         }
 
         // Narrow down root bracket.
         (f < 0. ? bk : ak) = xk;
 
-        if (x <= ak || x >= bk)
+        if (unlikely(x <= ak) || unlikely(x >= bk))
         {
             // Bisection step (ala Dekker's method).
             x = (ak + bk) * 0.5;
