@@ -1,7 +1,7 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2014 Leandro Nini <drfiemost@users.sourceforge.net>
  * Copyright 2007-2010 Antti Lankila
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,22 +25,43 @@
 namespace reSIDfp
 {
 
-typedef double(*Params)[6];
-
 /**
-*/
+ * Fritsch-Carlson monotone cubic spline interpolation.
+ *
+ * Based on the implementation from the wikipedia page:
+ * https://en.wikipedia.org/wiki/Monotone_cubic_interpolation
+ */
 class Spline
 {
+public:
+    typedef struct
+    {
+        double x;
+        double y;
+    } Point;
+
+private:
+    typedef double(*Params)[6];
+
 private:
     double* c;
     const int paramsLength;
     Params params;
 
+private:
+    /**
+     * Calculate the slope of the line crossing the given points.
+     */
+    inline double slope(const Point &a, const Point &b);
+
 public:
-    Spline(const double input[][2], int inputLength);
+    Spline(const Point input[], int inputLength);
     ~Spline() { delete [] params; }
 
-    void evaluate(double x, double* out);
+    /**
+     * Evaluate y and its derivative at given point x.
+     */
+    void evaluate(double x, Point &out);
 };
 
 } // namespace reSIDfp
